/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _887MP7JRSFILTER2DTYPE_H
#define _887MP7JRSFILTER2DTYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsFilter2DTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsFilter2DType_ExtInclude.h


#include "Mp7JrsFilterType.h"
class IMp7JrsDoubleMatrixType;
class IMp7JrsFilter2DType;
class IMp7JrsFilterType;
typedef Mp7JrsPtr< IMp7JrsDoubleMatrixType > Mp7JrsDoubleMatrixPtr;
typedef Mp7JrsPtr< IMp7JrsFilter2DType> Mp7JrsFilter2DPtr;
typedef Mp7JrsPtr< IMp7JrsFilterType > Mp7JrsFilterPtr;

/** 
 * Generated interface IMp7JrsFilter2DType for class Mp7JrsFilter2DType<br>
 * Located at: mpeg7-2004.xsd, line 7351<br>
 * Classified: Class<br>
 * Derived from: FilterType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsFilter2DType 
 :
        public IMp7JrsFilterType
{
public:
    // TODO: make these protected?
    IMp7JrsFilter2DType();
    virtual ~IMp7JrsFilter2DType();
    /**
     * Get xPhase attribute.
     * @return int
     */
    virtual int GetxPhase() const = 0;

    /**
     * Get xPhase attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistxPhase() const = 0;

    /**
     * Set xPhase attribute.
     * @param item int
     */
    virtual void SetxPhase(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate xPhase attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatexPhase() = 0;

    /**
     * Get yPhase attribute.
     * @return int
     */
    virtual int GetyPhase() const = 0;

    /**
     * Get yPhase attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistyPhase() const = 0;

    /**
     * Set yPhase attribute.
     * @param item int
     */
    virtual void SetyPhase(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate yPhase attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateyPhase() = 0;

        /** @name Sequence

         */
        //@{
    /**
     * Get Terms element.
     * @return Mp7JrsDoubleMatrixPtr
     */
    virtual Mp7JrsDoubleMatrixPtr GetTerms() const = 0;

    /**
     * Set Terms element.
     * @param item const Mp7JrsDoubleMatrixPtr &
     */
    // Mandatory			
    virtual void SetTerms(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;

        //@}
    /**
     * Get dim attribute.
<br>
     * (Inherited from Mp7JrsFilterType)
     * @return unsigned
     */
    virtual unsigned Getdim() const = 0;

    /**
     * Get dim attribute validity information.
     * (Inherited from Mp7JrsFilterType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existdim() const = 0;

    /**
     * Set dim attribute.
<br>
     * (Inherited from Mp7JrsFilterType)
     * @param item unsigned
     */
    virtual void Setdim(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate dim attribute.
     * (Inherited from Mp7JrsFilterType)
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatedim() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of Filter2DType.
     * Currently this type contains one child element of type "Mp7JrsNodePtr".

     * <ul>
     * <li>Terms</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsFilter2DType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsFilter2DType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsFilter2DType<br>
 * Located at: mpeg7-2004.xsd, line 7351<br>
 * Classified: Class<br>
 * Derived from: FilterType (Class)<br>
 * Defined in mpeg7-2004.xsd, line 7351.<br>
 */
class MP7JRS_EXPORT Mp7JrsFilter2DType :
        public IMp7JrsFilter2DType
{
    friend class Mp7JrsFilter2DTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsFilter2DType
     
     * @return Mp7JrsPtr< Mp7JrsFilterType >
     */
    virtual Mp7JrsPtr< Mp7JrsFilterType > GetBase() const;
    /*
     * Get xPhase attribute.
     * @return int
     */
    virtual int GetxPhase() const;

    /*
     * Get xPhase attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistxPhase() const;

    /*
     * Set xPhase attribute.
     * @param item int
     */
    virtual void SetxPhase(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate xPhase attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidatexPhase();

    /*
     * Get yPhase attribute.
     * @return int
     */
    virtual int GetyPhase() const;

    /*
     * Get yPhase attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistyPhase() const;

    /*
     * Set yPhase attribute.
     * @param item int
     */
    virtual void SetyPhase(int item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate yPhase attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateyPhase();

        /* @name Sequence

         */
        //@{
    /*
     * Get Terms element.
     * @return Mp7JrsDoubleMatrixPtr
     */
    virtual Mp7JrsDoubleMatrixPtr GetTerms() const;

    /*
     * Set Terms element.
     * @param item const Mp7JrsDoubleMatrixPtr &
     */
    // Mandatory			
    virtual void SetTerms(const Mp7JrsDoubleMatrixPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);

        //@}
    /*
     * Get dim attribute.
<br>
     * (Inherited from Mp7JrsFilterType)
     * @return unsigned
     */
    virtual unsigned Getdim() const;

    /*
     * Get dim attribute validity information.
     * (Inherited from Mp7JrsFilterType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existdim() const;

    /*
     * Set dim attribute.
<br>
     * (Inherited from Mp7JrsFilterType)
     * @param item unsigned
     */
    virtual void Setdim(unsigned item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate dim attribute.
     * (Inherited from Mp7JrsFilterType)
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatedim();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of Filter2DType.
     * Currently this type contains one child element of type "Mp7JrsNodePtr".

     * <ul>
     * <li>Terms</li>
     * </ul>
     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     * \par<code>A</code> 
     * - Select all child elements named <code>A</code>.
     * - In case of no element, create a new one.
     * - For collections: all subsequent XPath expressions are applied on all collection items (e.g. <code>A/B</code> query or create, childnodes <code>B</code> of all <code>A</code>s). 
     * \par<code>A[i]</code>
     * - Select the <code>i</code>th child element of all <code>A</code>.
     * - Please note, that XPath indexes start at 1.
     * - If there is no such <code>i</code>th element, try to create a new one at index <code>i</code>.
     * - Single elements require <code>i</code> to be 1 otherwise an exception is thrown.
     * - For collections, <code>i</code> must not exceed the number of existing items +1 (appending is allowed, filling gaps not).
     * - If the index is too high, a <code>MissingElementException</code> is thrown, and you can extract the last valid index. (Example: If a collection contains 2 elements and you are requesting the 4th element).
     * 
     * \par <code>A[\@xsi:type="T"]</code>
     *     - Select all child elements named <code>A</code> of type <code>T</code>.
     *     - Currently only typenames without namespace are recognised.
     *     - If there is no such element, create and append a new one.
     * \par <code>A[i][\@xsi:type="T"]</code>
     *     - Select the <code>i</code>th child element named <code>A</code>, which must be of type <code>T</code>.
     *     - If there is no such element at index <code>i</code>, a new one is created and inserted (appended).
     * \par <code>A[\@xsi:type="T"][i]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the <code>i</code>th one.
     *     - If there is no such element at index <code>i</code>, create and append one.
     * 
     * \par <code>A[last()]</code>
     *     - Select the last child element named <code>A</code>.
     *     - In case of no element, create a new one.
     * \par <code>A[last()][\@xsi:type="T"]</code>
     *     - Select the last child element named <code>A</code>, which must be of type <code>T</code>.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[\@xsi:type="T"][last()]</code>
     *     - Out of all child elements named <code>A</code> of type <code>T</code>, take the last one.
     *     - For collections: in case of no such type, create and append a new one.
     *     - For single elements: if an element with different type exists, return the empty list.
     * 
     * \par <code>A[position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> with its default type.
     *     - For collections: create and append the default element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[position() \> last()][\@xsi:type="T"]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - For collections: create and append the element.
     *     - For single elements: if any element exists, an exception is thrown.
     *
     * \par <code>A[\@xsi:type="T"][position() \> last()]</code>
     *     - Force creation of a child element named <code>A</code> of type <code>T</code>.
     *     - Behaves the same way as the previous expression.
     * 
     * 
     * Supported XPath expressions for GetFromXPath<br>
     * The same expressions like GetOrCreateFromXPath plus
     * \par <code>A[position() \< i] </code>
     *     - Select all child elements named <code>A</code> at indexes lower than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code>.
     *     - <code>i</code> must be greater than -1, otherwise an exception is thrown.
     * \par <code>A[position() \> i | position() \< j] </code>
     *     - Select all child elements named <code>A</code> at indexes greater than <code>i</code> and lower than <code>j</code>.
     *     - <code>i</code> and <code>j</code> must be greater than -1, otherwise an exception is thrown.
     *     - If <code>i</code> is greater than <code>j</code> then an empty list is returned.
     *
     * These three expressions can be combined with the <code>\@xsi:type</code> attribute to get a more restricted query.
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsFilter2DType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsFilter2DType();

protected:
    Mp7JrsFilter2DType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    int m_xPhase;
    bool m_xPhase_Exist;
    int m_xPhase_Default;
    int m_yPhase;
    bool m_yPhase_Exist;
    int m_yPhase_Default;

    // Base Class
    Mp7JrsPtr< Mp7JrsFilterType > m_Base;

    Mp7JrsDoubleMatrixPtr m_Terms;

// no includefile for extension defined 
// file Mp7JrsFilter2DType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _887MP7JRSFILTER2DTYPE_H

