/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/


// ClassType_H.template
#ifndef _2015MP7JRSSILENCETYPE_H
#define _2015MP7JRSSILENCETYPE_H

#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOM.hpp>

#include "Mp7JrsDefines.h"
#include "Mp7JrsFactoryDefines.h"
#include "Mp7JrsSilenceTypeFactory.h"
#include "Mp7JrsNode.h"
#include "Mp7JrsConvert.h"


// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtInclude.h


#include "Mp7JrsAudioDType.h"
#include "Mp7JrsDType.h"
#include "Mp7JrsMpeg7BaseType.h"
class IMp7JrsAudioDType;
class IMp7JrsintegerVector;
class IMp7JrsSilenceType;
class IMp7JrszeroToOneType;
typedef Mp7JrsPtr< IMp7JrsAudioDType > Mp7JrsAudioDPtr;
typedef Mp7JrsPtr< IMp7JrsintegerVector > Mp7JrsintegerVectorPtr;
typedef Mp7JrsPtr< IMp7JrsSilenceType> Mp7JrsSilencePtr;
typedef Mp7JrsPtr< IMp7JrszeroToOneType > Mp7JrszeroToOnePtr;

/** 
 * Generated interface IMp7JrsSilenceType for class Mp7JrsSilenceType<br>
 * Located at: mpeg7-2004.xsd, line 1830<br>
 * Classified: Class<br>
 * Derived from: AudioDType (Class)<br>
 */
class MP7JRS_EXPORT IMp7JrsSilenceType 
 :
        public IMp7JrsAudioDType
{
public:
    // TODO: make these protected?
    IMp7JrsSilenceType();
    virtual ~IMp7JrsSilenceType();
    /**
     * Get confidence attribute.
     * @return Mp7JrszeroToOnePtr
     */
    virtual Mp7JrszeroToOnePtr Getconfidence() const = 0;

    /**
     * Get confidence attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existconfidence() const = 0;

    /**
     * Set confidence attribute.
     * @param item const Mp7JrszeroToOnePtr &
     */
    virtual void Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate confidence attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateconfidence() = 0;

    /**
     * Get minDurationRef attribute.
     * @return XMLCh *
     */
    virtual XMLCh * GetminDurationRef() const = 0;

    /**
     * Get minDurationRef attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistminDurationRef() const = 0;

    /**
     * Set minDurationRef attribute.
     * @param item XMLCh *
     */
    virtual void SetminDurationRef(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate minDurationRef attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateminDurationRef() = 0;

    /**
     * Get channels attribute.
<br>
     * (Inherited from Mp7JrsAudioDType)
     * @return Mp7JrsintegerVectorPtr
     */
    virtual Mp7JrsintegerVectorPtr Getchannels() const = 0;

    /**
     * Get channels attribute validity information.
     * (Inherited from Mp7JrsAudioDType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existchannels() const = 0;

    /**
     * Set channels attribute.
<br>
     * (Inherited from Mp7JrsAudioDType)
     * @param item const Mp7JrsintegerVectorPtr &
     */
    virtual void Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT) = 0;


    /**
     * Invalidate channels attribute.
     * (Inherited from Mp7JrsAudioDType)
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatechannels() = 0;

    /**
     * Gets or creates Mp7JrsNodePtr child elements of SilenceType.
     * Currently just supports rudimentary XPath expressions as SilenceType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL) = 0;

    
// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtMethodDef.h


// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtPropDef.h

};




/*
 * Generated class Mp7JrsSilenceType<br>
 * Located at: mpeg7-2004.xsd, line 1830<br>
 * Classified: Class<br>
 * Derived from: AudioDType (Class)<br>
 * Defined in mpeg7-2004.xsd, line 1830.<br>
 */
class MP7JRS_EXPORT Mp7JrsSilenceType :
        public IMp7JrsSilenceType
{
    friend class Mp7JrsSilenceTypeFactory; // constructs objects

public:
    /*
     * Retrieves the base class of Mp7JrsSilenceType
     
     * @return Mp7JrsPtr< Mp7JrsAudioDType >
     */
    virtual Mp7JrsPtr< Mp7JrsAudioDType > GetBase() const;
    /*
     * Get confidence attribute.
     * @return Mp7JrszeroToOnePtr
     */
    virtual Mp7JrszeroToOnePtr Getconfidence() const;

    /*
     * Get confidence attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existconfidence() const;

    /*
     * Set confidence attribute.
     * @param item const Mp7JrszeroToOnePtr &
     */
    virtual void Setconfidence(const Mp7JrszeroToOnePtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate confidence attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidateconfidence();

    /*
     * Get minDurationRef attribute.
     * @return XMLCh *
     */
    virtual XMLCh * GetminDurationRef() const;

    /*
     * Get minDurationRef attribute validity information.
     * @return true if attribute value is valid, false if not.
     */
    virtual bool ExistminDurationRef() const;

    /*
     * Set minDurationRef attribute.
     * @param item XMLCh *
     */
    virtual void SetminDurationRef(XMLCh * item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate minDurationRef attribute.
     
     * (Validate it again by setting a value)
     */
    virtual void InvalidateminDurationRef();

    /*
     * Get channels attribute.
<br>
     * (Inherited from Mp7JrsAudioDType)
     * @return Mp7JrsintegerVectorPtr
     */
    virtual Mp7JrsintegerVectorPtr Getchannels() const;

    /*
     * Get channels attribute validity information.
     * (Inherited from Mp7JrsAudioDType)
     * @return true if attribute value is valid, false if not.
     */
    virtual bool Existchannels() const;

    /*
     * Set channels attribute.
<br>
     * (Inherited from Mp7JrsAudioDType)
     * @param item const Mp7JrsintegerVectorPtr &
     */
    virtual void Setchannels(const Mp7JrsintegerVectorPtr & item, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT);


    /*
     * Invalidate channels attribute.
     * (Inherited from Mp7JrsAudioDType)
     
     * (Validate it again by setting a value)
     */
    virtual void Invalidatechannels();

    /*
     * Gets or creates Mp7JrsNodePtr child elements of SilenceType.
     * Currently just supports rudimentary XPath expressions as SilenceType contains no child elements of type "Mp7JrsNodePtr" at all.

     * 
     * Supported XPath expressions for GetOrCreateFromXPath
     * 
     * \par<code>/</code>
     *  - Get the root of this node.
     * \par<code>..</code>
     *  - This feature is experimental.
     *  - Get the ancestor (parent node), or an empty list if the node already is the root node.
     * 
     *
     * @param xpath  the xpath expression to get or create.
     * @param client the ID in case of locking.
     * @return the appropriate subelement node list. Can be empty in case of incompatible name/type/index settings.
     */
    // xpath: mandatory
    // client: optional
	virtual Mp7JrsNodeEnum GetOrCreateFromXPath(const XMLCh *xpath, Mp7JrsClientID client = MP7JRS_UNDEFINED_CLIENT, Mp7JrsXPathCreation flag = MP7JRS_FORCECREATE, Mp7JrsXPathParseContext * context = NULL);

    

    /* Get the class name.
     * @return The name of this class.
     */
    virtual XMLCh * ToText() const;

    /* Parse this type.
     * @param txt buffer to parse from.
     * @return true if parsing was successful, false otherwise.
     */
    virtual bool Parse(const XMLCh * const txt);

    /* Serialialize this object to a DOM tree.
     * @param doc DOM document.
     * @param parent Parent node in DOM tree.
     */
    virtual void Serialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement* parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement** newElem = NULL);

    /* Deserialize this object from a DOM tree.
     * @param doc DOM document.
     * @param parent Parent element in DOM tree.
     * @param current Current element in DOM tree.
     */
    virtual bool Deserialize(XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument * doc, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement * parent, XERCES_CPP_NAMESPACE_QUALIFIER DOMElement ** current);

    /* Creates and inserts a Childnode of the specified Type
     * @param type The tag type of the child to be created (tag name).
     * @param xsitype The xsi type of the child to be created if available.
     * @return the created child with the correct dynamic type for the specified type.
     */
    virtual Mp7JrsNodePtr CreateChild(const XMLCh * const elementname, const XMLCh * const xsitype);
         
// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtMyMethodDef.h


public:

    virtual ~Mp7JrsSilenceType();

protected:
    Mp7JrsSilenceType();
    virtual void DeepCopy(const Mp7JrsNodePtr &original);

    virtual Mp7JrsNodeEnum GetAllChildElements() const;

    virtual Mp7JrsNodePtr GetBaseRoot();
    virtual const Mp7JrsNodePtr GetBaseRoot() const;

private:
    Mp7JrszeroToOnePtr m_confidence;
    bool m_confidence_Exist;
    Mp7JrszeroToOnePtr m_confidence_Default;
    XMLCh * m_minDurationRef;
    bool m_minDurationRef_Exist;

    // Base Class
    Mp7JrsPtr< Mp7JrsAudioDType > m_Base;


// no includefile for extension defined 
// file Mp7JrsSilenceType_ExtMyPropDef.h


public:
    virtual void Init(); // Called by inheritees to recursively update m_myself
protected:
    void Cleanup();
};

#endif // _2015MP7JRSSILENCETYPE_H

