/*****************************************************************************/
/* Copyright (C) 2003-2012, JOANNEUM RESEARCH Forschungs Ges.m.b.H.          */
/* Steyrergasse 17, A-8010 Graz, Austria.                                    */
/*                                                                           */
/*  This file is part of the JRS MPEG-7 Library.                             */
/*  http://mpeg-7.joanneum.at                                                */
/*                                                                           */
/*  JRS MPEG-7 Library is free software: you can redistribute it and/or      */
/*  modify it under the terms of the GNU Lesser General Public License       */
/*  (LGPL) as published by the Free Software Foundation, either version 3    */
/*  of the License, or (at your option) any later version.                   */
/*                                                                           */
/*  JRS MPEG-7 Library is distributed in the hope that it will be useful,    */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of           */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             */
/*  GNU Lesser General Public License for more details.                      */
/*                                                                           */
/*  You should have received a copy of the GNU Lesser General Public License */
/*  along with this library. If not, see <http://www.gnu.org/licenses/>.     */
/*****************************************************************************/



/*! \mainpage The Mp7Jrs library
 *
 * \section intro Introduction
 *
 * MPEG-7 library, version 2004 
 *
 * \section about About this API documentation
 *
 * This set of HTML pages is an automatically generated API documentation of the Mp7Jrs classes. It is intended as a reference for a developer using Mp7Jrs. If you are looking for an overview description of the library, instructions on how to setup a project using Mp7Jrs and some commented examples, please look at the document located in the /doc subdirectory of the Mp7Jrs distribution.
 * <br>
 * This API documentation contains the following:
 * <ul>
 * <li>class descriptions, including the extension base type and the schema file and line number where they are defined</li>
 * <li>descriptions of methods, including a reference to the class from which they are inherited</li>
 * <li>if the type of parameters or return values is abstract, a list of allowed types is provided</li>
 * <li>for collections, a reference to the local type which is the default type of collection elements</li>
 * </ul>
 *
 
 * \section extensions Extensions
 
 			
 * Currently there are no extensions created with this version of Mp7Jrs.
 * All you need to extend types of the core library, is an XML schema,
 * which must be - of course - an extension of the core schema and the extented types
 * must have their own namespace(s). Then an additional library can easily be created
 * along with the core lib.
 *	
 * For more information about this, please contact <a href="http://www.joanneum.at/en/digital/avm/products-solutions-services/mpeg-7-library.html">DIGITAL-AVM (formerly IIS)</a> at JOANNEUM RESEARCH.
 * 
 * \section news Whats new?
 *
 * Version 2.5:<br>
 * Links against bugfixed XercesC 3.3.1 now.
 * <br>Builds on 64 bit platforms (Win64 VC9, LINUX 64 bit, MacOS 10.5 64 bit).
 * <br>With adjusted CMAKE file to build and install libs and doc on LINUX the more natural way.
 * <br>Example: <code>cmake -G "Unix Makefiles" . , make, make doc, sudo make install</code>.
 *
 * Version 2.4:<br>
 * Switched to MPEG-7 standard 2004 (using namespace xmlns:mpeg7="urn:mpeg:mpeg7:schema:2004")
 * Fixed a lot of bugs - mostly concerning extension libraries. 
 *
 * Version 2.3:<br>
 * Full XML namespace support. Mp7Jrs serialises XML with full namespace declaration in the root element.
 * If you define a default namespace, no prefix is used. Xerces V.2.7 does not allow both - a default namespace beeing
 * declared the second time with a prefix as well. So you can either choose between the default namespace or the prefixed version
 * of your (default) namespace. Having all namespaces prefixed is very handy for XSLTs.
 * To do so, use <code>myArchive.SetUsePrefixForDefaultNS(true);</code>
 *
 * On the contrary, if you generate MPEG-7 fragments with your own or other third-party tools, and read them with 
 * Mp7Jrs library, you must provide a proper namespace declaration in the fragment root
 * <br>e.g.: <code><mpeg7:MyFragment xmlns:mpeg7="urn:mpeg:mpeg7:schema:2004" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">My_Data</mpeg7:MyFragment></code>
 * to read it properly. If you don't use XSI types, you can suppress the declaration of XSI namepace via 
 * <code>myArchive.SetForceNamespaceDeclaration(false);</code>
 
 *
 * Version 2.2:<br>
 * ATTENTION: A stunning new method for accessing nodes has been introduced.
 * Instead of dealing with difficult intermediate types (and a huge \#include section), it is strongly recommended to use 
 * <code>GetFromXPath()</code> or <code>GetOrCreateFromXPath()</code> methods now.
 * So one single call retrieves a list of desired nodes or creates a complete path with one or more nodes.
 *
 * Example:<br>
 * <code>root.GetFromXPath("A/B/C[last()]/D[3]/E[@xsi:type="Desired"]</code> to get all desired nodes with this type.
 * or:<br>
 * <code>rootnode.GetOrCreateFromXPath("A/B/C[last()]/D[3]/E[@xsi:type="Desired"]</code> creates a node with this type (if there was none before).
 *
 * For a detailed list of supported XPath expressions please read the class documentation.<br>
 * These two methods should be sufficient to handle your requests properly.
 *
 *
 * ATTENTION: The methods <code>Archive::ToWBuffer()</code> and <code>Archive::ToBuffer()</code> are using 
 * Xerces heap now. You *MUST* call the appropriate <code>Archive::DeleteBuffer()</code> methods now
 * , otherwise memory errors will occur! 
 *
 * ATTENTION: The macros for <code>GetObjectType</code> and <code>GetTypeId</code> have been removed,
 * in order to get rid of nasty library problems with existing MS libs! If you need these definitions,
 * redefine them in your source code.
 *
 * ATTENTION: The macro <code>GetClassType</code> has been removed. Use the correspondend <code>GetTypeName()</code>
 * method instead. If you need this definition, please do it in your source code.
 *
 * ATTENTION: The method <code>GetNextSibling()</code> was renamed to <code>GetNextAkin()</code> because
 * of interferences with a macro from MS XML library...
 * If you need this definition, please do it in your source code.
 *
 * Other changes:
 * <ul>
 * <li>Subelements of type string are now created automatically (as empty string) if they are mandatory.
 * <li>Abstract classes automatically handle the <code>UseTypeAttribute</code> flag, needed for correct serialisation.</li>
 * <li>Interface setter methods additionally create VS7 IntelliSense comments for mandatory elements.</li>
 * <li>Library version string can be queried with <code>GetVersion()</code> method.</li>
 * </ul>
 
 */
